<?php
/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 ***************************************************************************/

define(MENCODER, "mencoder");
define(CONVERT, "convert");

class media_detector
{
        var $orig_dir;
        var $medium_dir;
        var $small_dir;
        var $album_id;
        var $processing;
        var $processing_ok;
        var $last_processed_file;

        var $medias = array();
        var $supported_formats = array();

	//
	// Constructor
	//
	function media_detector($orig_dir, $album_id, $process = true)
	{
                global $configs;
                $this->orig_dir = $orig_dir;
                $this->medium_dir = $configs['medium_size'] . '/';
                $this->thumbnail_dir = $configs['thumbnail_size'] . '/';
                $this->album_id = $album_id;
	        $this->supported_formats[0] = ".jpg";
        	$this->supported_formats[1] = ".jpeg";
	        $this->supported_formats[2] = ".avi";
                $this->processing_ok = true;

                // get files from the directory
                if (is_dir($this->orig_dir))
                {
                        if ($dh = opendir($this->orig_dir))
                        {
                                while (($file = readdir($dh)) !== false)
                                {
                                        // filter all dirs and hided
                                        // files away
                                        if (is_dir($this->orig_dir . $file))
                                            continue;

                                        // check that the file format is supported
                                        $supported = false;
                                        for ($i = 0; $i < count($this->supported_formats); $i++)
                                        {
                                                if (strstr($file, $this->supported_formats[$i]) ||
                                                    strstr($file, strtoupper($this->supported_formats[$i])))
                                                {
                                                        $supported = true;
                                                        break;
                                                }
                                        }
                                        if (!$supported) continue;

                                        // check that medium size image exists
                                        if (!is_file($this->get_medium_filename($file)))
                                        {
                                                if ($process)
                                                {
                                                	// create a medium size image
                                                	$this->show_please_wait($file);
                                                	if ($this->create_thumbnail($file,
                                                			    $this->medium_dir,
                                                                            $configs['medium_size']) == false)
                                                	{
                                                                // processing failed
                                                        	$this->processing_ok = false;
                                                        	continue;
                                                	}
                                                }
                                                else continue;
                                        }

                                        // check that it has a thumbnail
                                        if (!is_file($this->get_thumbnail_filename($file)))
                                        {
                                                if ($process)
                                                {
	                                        	// create a thumbnail
                                                	$this->show_please_wait($file);
                                                	if ($this->create_thumbnail($file,
                                                                            	$this->thumbnail_dir,
                                                                            	$configs['thumbnail_size']) == false)
                                                	{
                                                                // processing failed
                                                        	$this->processing_ok = false;
                                                        	continue;
                                                	}
                                                }
                                                else continue;
                                        }

                                        // all checks ok! accept the file
                                        $this->medias[] = $file;
                                }
                                closedir($dh);
                        }
                }
		sort($this->medias);
		$this->add_missing_medias_into_db();
        }

        function add_missing_medias_into_db()
        {
                global $db;
                // now we have the files
                for ($i = 0; $i < count($this->medias); $i++)
                {
                        $sql = "SELECT id FROM medias
                                WHERE filename = '" . $this->medias[$i] . "'
                                AND album_id = '" . $this->album_id . "'";

                        if (!($result = $db->sql_query($sql)))
                        {
                                msg_die("Error fetching media data", '', __FILE__, __LINE__, $sql);
                        }

                        if ($db->sql_numrows() == 0)
                        {
                                // media item not found from db, add it
                                // generate id for the media
                                $media_id = md5(uniqid(""));

                                // let's add the media into db
                                $sql = "INSERT INTO medias
                                        (id, filename, album_id, rate, created)
                                        VALUES ('$media_id', '" . $this->medias[$i] . "', '" . $this->album_id . "', -1, now())";

                                if (!($result = $db->sql_query($sql)))
                                {
                                        msg_die("Error inserting media data", '', __FILE__, __LINE__, $sql);
                                }
                                //echo "debug: " . $this->medias[$i] . " inserted into DB!!!<br>";
                        }
                        //echo $i + 1 . ": " . $this->medias[$i];
                        //echo "<br>";

                }
        }

        function get_thumbnail_filename($file)
        {
                $tn = $this->thumbnail_dir;
                if (strstr($file, ".avi") ||
                    strstr($file, ".AVI"))
                {
                        $tn .= substr_replace($file, '.jpg', strlen($file) - 4, 4);
                }
                else
                {
                        $tn .= $file;
                }
                return $tn;
        }

        function get_medium_filename($file)
        {
                $medium = $this->medium_dir;
                $medium .= $file;
                return $medium;
        }

        function create_thumbnail($file, $directory, $size)
        {
                global $configs;

                $target = '';
                // at first, check that the directory exists
                if (!is_dir($directory))
                {
                        if (mkdir($directory) == false)
                        {
                        	echo "Could not create directory " . $directory . "<br>";
                        	return false;
                        }
                	chmod($directory, $configs['media_dir_permissions']);
                }

                switch ($size)
                {
                case $configs['medium_size']:
                	$target = $this->get_medium_filename($file);
                        break;

                case $configs['thumbnail_size']:
                        $target = $this->get_thumbnail_filename($file);
                        break;
                }

                if (strstr($file, $this->supported_formats[2]) ||
                    strstr($file, strtoupper($this->supported_formats[2])))
                {
                        // allright, we have avi video clip here
                        // at least some challenge to handle it :)
                        switch ($size)
                        {
                        case $configs['medium_size']:
                                encode_divx($file, $target);
                                break;

                        case $configs['thumbnail_size']:
				make_thumbnail_from_video($file, $target);
                                break;
                        }
                }
                else
                {
                        // plarf, its just jpeg image, lets scale
                        // it using imagemagick toolkit
                        scale_image($file, $target, $size);
                }
                if (!is_file($target))
                {
                        //echo "creation of file '" . $target . "' failed.<br>";
                        return false;
                }
                return true;
        }

        function get_media_count()
        {
                return count($this->medias);
        }

        function get_directory()
        {
                return $this->directory();
        }

        function get_media_info($i)
        {
                global $db;
                if ($i < 0 || $i >= count($this->medias))
                        return false; // out of scale

                $sql = "SELECT * FROM medias
                        WHERE filename = '" . $this->medias[$i] . "'
                        AND album_id = '" . $this->album_id . "'";

        	if (!($result = $db->sql_query($sql)))
                {
                	msg_die("Error fetching media data", '', __FILE__, __LINE__, $sql);
                }

                if (!($row = $db->sql_fetchrow($result)))
                {
                        msg_die("some fancy error", '', __FILE__, __LINE__, $sql);
                }
                return $row;
        }

        function get_i_by_id($id)
        {
                $count_medias = count($this->medias);
                for ($i = 0; $i < $count_medias; $i++)
                {
                	$media_info = $this->get_media_info($i);
                        if ($media_info['id'] == $id)
                                return $i;
                }
                return false;
        }

        function show_please_wait($file)
        {
                if ($file != $this->last_processed_file)
                {
                	if (!$this->processing)
                	{
                        	echo "<html><head><title>LiveAlbum - Processing</title></head><body>";
                        	echo "<h2>Please wait</h2>";
                        	$this->processing = true;
                	}
                	echo "Processing " . $file . " ...<br>";
                        flush();
                        $this->last_processed_file = $file;
                }
        }
}

?>
